package tekito;

import java.net.URL;
import java.util.ResourceBundle;
import java.util.stream.IntStream;
import javafx.beans.property.ReadOnlyStringWrapper;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Alert;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.text.Font;

public class FXMLDocumentController implements Initializable {
    private static final int NUM_COLUMNS = 16;
    @FXML
    private ChoiceBox fontChoiceBox;
    @FXML
    private TableView gryphTable;
    @FXML
    private TableColumn<Integer, String> rowHeader;

    private final ObservableList<String> fonts = FXCollections.observableArrayList();
    private final ObservableList<Integer> rowIndices = FXCollections.observableArrayList();
    private java.awt.Font currentFont;

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        gryphTable.setItems(rowIndices);
        gryphTable.setColumnResizePolicy(TableView.CONSTRAINED_RESIZE_POLICY);        
        rowHeader.setCellValueFactory(rowIndex -> new ReadOnlyStringWrapper(String.format("%03X0", rowIndex.getValue())));
        IntStream.range(0, NUM_COLUMNS).forEach(index -> {
            TableColumn<Integer, String> column = new TableColumn<>(String.format("%X", index));
            column.setCellValueFactory(rowIndex -> {
                int codePoint = rowIndex.getValue() * NUM_COLUMNS + index;
                if (currentFont.canDisplay(codePoint)) {
                    return new ReadOnlyStringWrapper(new String(new int [] {codePoint}, 0, 1));
                } else {
                    return null;
                }
            });
            gryphTable.getColumns().add(column);
        });        
        
        fonts.addAll(Font.getFamilies());
        fontChoiceBox.setItems(fonts);
        fontChoiceBox.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            String name = (String) newValue;
            if ("System".equals(Font.font(name).getFamily()) && !"System".equals(name)) {
                // HG系のフォントなど、うまく設定できずデフォルトのSystemフォントになってしまう場合があるのでその対処
                new Alert(Alert.AlertType.WARNING, name + "を表示できません。").showAndWait();
                rowIndices.clear();
                return;
            }
            currentFont = new java.awt.Font(name, 0, 24);
            gryphTable.setStyle("-fx-font-family: '" + name + "'");
            rowIndices.clear();
            IntStream.range(0, 0xF000).forEach(index -> rowIndices.add(index));        
        });
        fontChoiceBox.getSelectionModel().select(0);
    }
}
